@@ -16,7 +16,9 @@ import os
 import pprint
 import random
 import re
+import shlex
 import string
+import subprocess
 import xml
 
 
@@ -41,6 +43,12 @@ def logline(*args):
     msg = '%s %s' % (cstr(caller, 'yellow'), args[0])
     logging.warning(msg, *args[1:])
 
+def shell(cmd, cwd=None):
+    if type(cmd) not in (tuple, list):
+        cmd = shlex.split(cmd)
+    output = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=False, cwd=cwd).communicate()[0]
+    return output.decode('utf-8').strip()
+
 def force_int(string):
     if type(string) in (int, int): return int(string)
     int_re = re.compile('^.*?(\d+).*?$')
@@ -48,8 +56,13 @@ def force_int(string):
     if not match: return None
     return int(match.group(1))
 
-def md5hex(val):
-    return hashlib.md5(val).hexdigest()
+def md5hex(val=None, encoding=None):
+    if val:
+        if encoding:
+            val = val.encode(encoding)
+        return hashlib.md5(val).hexdigest()
+    else:
+        return hashlib.md5().hexdigest()
 
 def html_unescape(val):
     return xml.sax.saxutils.unescape(val, {'&quot;': '"'})
